@@ -74,7 +74,7 @@ def putdown_fork(client, actor, fork):
     try:
         self.not_available_forks.remove(fork)
     except KeyError:
-        self.log.error('Putting down a fork which was already available')    
+        self.logger.error('Putting down a fork which was already available')    
 
 @command(commands_set=philosophers_cmommands)
 def pickup_fork(client, actor, fork_right):
@@ -102,68 +102,68 @@ class DiningPhilosophers(pulsar.Application):
         
     def worker_task(self, philosopher):
         # Task performed at each loop in the philosopher I/O loop
-        local = philosopher.local
-        eaten = local.eaten or 0
-        forks = local.forks
-        started_waiting = local.started_waiting or 0
+        params = philosopher.params
+        eaten = params.eaten or 0
+        forks = params.forks
+        started_waiting = params.started_waiting or 0
         if forks:
             max_eat_period = 2*self.cfg.eating_period
             if len(forks) == 2:
-                local.thinking = 0
+                params.thinking = 0
                 eaten += 1
-                philosopher.log.info("%s eating... So far %s times",
-                                     philosopher.name, eaten)
+                philosopher.logger.info("%s eating... So far %s times",
+                                        philosopher.name, eaten)
                 try:
                     time.sleep(max_eat_period*random.random())
                 except IOError:
                     pass
-                local.eaten = eaten
+                params.eaten = eaten
                 self.release_forks(philosopher)
             elif len(forks) == 1:
                 waiting_period = 2*self.cfg.waiting_period*random.random()
                 if started_waiting == 0:
-                    local.started_waiting = time.time()
+                    params.started_waiting = time.time()
                 elif time.time() - started_waiting > waiting_period:
                     self.release_forks(philosopher)
                 else:
                     self.check_forks(philosopher)
             elif len(forks) > 2:
-                philosopher.log.critical('%s has more than 2 forks!!!',
-                                         philosopher.name)
+                philosopher.logger.critical('%s has more than 2 forks!!!',
+                                            philosopher.name)
                 self.release_forks(philosopher)
         else:
-            thinking = local.thinking or 0
+            thinking = params.thinking or 0
             if not thinking:
-                philosopher.log.warn('%s thinking...', philosopher.name)
-            local.thinking = thinking + 1
+                philosopher.logger.warn('%s thinking...', philosopher.name)
+            params.thinking = thinking + 1
             self.check_forks(philosopher)
         
     def check_forks(self, philosopher):
         '''The philosopher has less than two forks. Check if forks are
 available.'''
-        right_fork = philosopher.local.number
+        right_fork = philosopher.params.number
         philosopher.send(philosopher.monitor, 'pickup_fork', right_fork)\
                    .add_callback_args(self.got_fork, philosopher)
     
     def release_forks(self, philosopher):
-        forks = philosopher.local.forks
-        philosopher.local.forks = []
-        philosopher.local.started_waiting = 0
+        forks = philosopher.params.forks
+        philosopher.params.forks = []
+        philosopher.params.started_waiting = 0
         for fork in forks:
-            philosopher.log.debug('Putting down fork %s', fork)
+            philosopher.logger.debug('Putting down fork %s', fork)
             philosopher.send(philosopher.monitor, 'putdown_fork', fork)
     
     def got_fork(self, fork, philosopher):
         if fork:
-            forks = philosopher.local.forks
+            forks = philosopher.params.forks
             if fork in forks:
-                philosopher.log.error('Got fork %s which I already have' % fork)
+                philosopher.logger.error('Got fork %s which I already have', fork)
             else:
-                philosopher.log.debug('Got fork %s.' % fork)
+                philosopher.logger.debug('Got fork %s.', fork)
                 forks.append(fork)
     
     def actorparams(self, monitor, params):
-        number = len(monitor.MANAGED_ACTORS) + len(monitor.SPAWNING_ACTORS) + 1
+        number = len(monitor.managed_actors) + len(monitor.spawning_actors) + 1
         name = 'Philosopher %s' % number
         params.update({'name': name,
                        'number': number,
