@@ -5,7 +5,7 @@ from inspect import isfunction
 
 import pulsar
 from pulsar import Actor, make_async, safe_async, is_failure, HaltServer,\
-                     Monitor, loop_timeout, Deferred, get_actor
+                     Monitor, loop_timeout, Deferred, get_actor, async
 from pulsar.async.defer import pickle
 from pulsar.utils import event
 from pulsar.utils.log import LogInformation
@@ -53,7 +53,7 @@ the :class:`Application` associated with *name* if available. If not in the
 :class:`Arbiter` domain it returns nothing.'''
     actor = get_actor()
     if actor and actor.is_arbiter():
-        monitor = actor._monitors.get(name)
+        monitor = actor.monitors.get(name)
         if monitor:
             return getattr(monitor, 'app', None) 
     
@@ -71,19 +71,13 @@ to the underlying :class:`Application`.'''
 
     def handle_task(self):
         if self.information.log():
-            self.log.info('Processed {0} requests'.format(self.nr))
+            self.logger.info('Processed %s requests', self.info.nr)
         try:
             self.cfg.worker_task(self)
         except:
             pass
         self.app.worker_task(self)
 
-    def configure_logging(self, config=None):
-        # Delegate to application
-        self.app.configure_logging(config=config)
-        self.loglevel = self.app.loglevel
-        self.setlog()
-
 
 class Worker(ApplicationHandlerMixin, Actor):
     """\
@@ -103,15 +97,10 @@ It provides two new methods inherited from :class:`ApplicationHandlerMixin`.
     The application handler obtained from :meth:`Application.handler`.
 
 """
-    @property
-    def class_code(self):
-        return 'worker %s' % self.app.name
-    
     def on_init(self, app=None, **kwargs):
         self.app = app
-        self.cfg = app.cfg
-        self.max_requests = self.cfg.max_requests or sys.maxsize
         self.information = LogInformation(self.cfg.logevery)
+        app.worker_init(self)
         self.app_handler = app.handler()
         return kwargs
 
@@ -145,28 +134,16 @@ class ApplicationMonitor(ApplicationHandlerMixin, Monitor):
     '''A specialized :class:`Monitor` implementation for managing
 pulsar subclasses of :class:`Application`.
 '''
-    # For logging name
-    @property
-    def class_code(self):
-        return 'monitor %s' % self.app.name
+    actor_class = Worker
     
     def on_init(self, app=None, **kwargs):
         self.app = app
-        self.cfg = app.cfg
-        self.max_requests = 0
-        arbiter = pulsar.arbiter()
-        # Set the arbiter config object if not available
-        if not arbiter.cfg:
-            arbiter.cfg = app.cfg
-        self.max_requests = None
         self.information = LogInformation(self.cfg.logevery)
         app.monitor_init(self)
         if not self.cfg.workers:
             self.app_handler = app.handler()
         else:
             self.app_handler = app.monitor_handler()
-        kwargs['actor_class'] = Worker
-        kwargs['num_actors'] = app.cfg.workers
         return super(ApplicationMonitor, self).on_init(**kwargs)
 
     ############################################################################
@@ -186,6 +163,7 @@ pulsar subclasses of :class:`Application`.
         if not self.cfg.workers:
             yield self.handle_task()
 
+    @async()
     def on_stop(self):
         if not self.cfg.workers:
             yield self.app.worker_stop(self)
@@ -201,9 +179,6 @@ pulsar subclasses of :class:`Application`.
         except:
             pass
 
-    def clean_up(self):
-        self.worker_class.clean_arbiter_loop(self,self.ioloop)
-
     def actorparams(self):
         '''Override the :meth:`Monitor.actorparams` method to
 updated actor parameters with information about the application.
@@ -212,27 +187,20 @@ updated actor parameters with information about the application.
     spawn method when creating new actors.'''
         p = Monitor.actorparams(self)
         app = self.app
-        impl = app.cfg.concurrency
-        if impl == 'thread':
+        if app.cfg.concurrency == 'thread':
             app = pickle.loads(pickle.dumps(app))
         p.update({'app': app,
-                  'timeout': app.cfg.timeout,
-                  'loglevel': app.loglevel,
-                  'max_concurrent_requests': app.cfg.backlog,
-                  'concurrency': impl,
-                  'name':'{0}-worker'.format(app.name)})
+                  'name': '{0}-worker'.format(app.name)})
         return app.actorparams(self, p)
 
     def on_info(self, info):
-        info['actor'].update({'default_timeout': self.cfg.timeout,
-                              'max_requests': self.cfg.max_requests})
         if not self.cfg.workers:
             return self.app.on_info(self, info)
         else:
             return info
 
 
-class Application(pulsar.LogginMixin):
+class Application(pulsar.Pulsar):
     """An application interface for configuring and loading
 the various necessities for any given server or distributed application running
 on :mod:`pulsar` concurrent framework.
@@ -317,7 +285,6 @@ These are the most important facts about a pulsar :class:`Application`
     cfg = {}
     _app_name = None
     description = None
-    mid = None
     epilog = None
     cfg_apps = None
     config_options_include = None
@@ -337,6 +304,7 @@ These are the most important facts about a pulsar :class:`Application`
                  can_kill_arbiter=None,
                  parse_console=True,
                  commands_set=None,
+                 cfg=None,
                  **kwargs):
         '''Initialize a new :class:`Application` and add its
 :class:`ApplicationMonitor` to the class:`pulsar.Arbiter`.
@@ -358,7 +326,9 @@ These are the most important facts about a pulsar :class:`Application`
             self.commands_set = commands_set
         self.script = script
         self.python_path()
-        params = self.cfg.copy() if self.cfg else {}
+        params = cfg or {}
+        if self.cfg:
+            params.update(self.cfg)
         params.update(kwargs)
         self.callable = callable
         self.load_config(argv, version, parse_console, params)
@@ -367,21 +337,24 @@ These are the most important facts about a pulsar :class:`Application`
     def __call__(self, actor=None):
         if actor is None:
             actor = get_actor()
-        if not self.mid and (not actor or actor.is_arbiter()):
+        monitor = None
+        if actor and actor.is_arbiter():
+            monitor = actor.monitors.get(self.name)
+        if monitor is None and (not actor or actor.is_arbiter()): 
             # Add events
             self.local.events = {'on_start': Deferred(), 'on_stop': Deferred()}
             self.cfg.on_start()
             self.configure_logging()
             event.fire('ready', sender=self)
+            arbiter = pulsar.arbiter(cfg=self.cfg)
             if self.on_config() is not False:
-                arbiter = pulsar.arbiter(self.cfg.daemon)
                 monitor = arbiter.add_monitor(self.monitor_class,
                                               self.name,
                                               app=self,
+                                              cfg=self.cfg,
                                               ioqueue=self.ioqueue)
-                self.mid = monitor.aid
                 if self.commands_set:
-                    monitor.commands_set.update(self.commands_set)
+                    monitor.impl.commands_set.update(self.commands_set)
         events = self.events
         if events:
             return events['on_start']
@@ -398,11 +371,6 @@ These are the most important facts about a pulsar :class:`Application`
     def name(self):
         '''Application name, It is unique and defines the application.'''
         return self._name
-    
-    @property
-    def monitor(self):
-        if self.mid:
-            return pulsar.arbiter()._monitors.get(self.mid)
 
     @property
     def events(self):
@@ -412,8 +380,14 @@ These are the most important facts about a pulsar :class:`Application`
         return self.name
 
     def __str__(self):
-        return self.name
+        return self.__repr__()
 
+    @property
+    def monitor(self):
+        actor = get_actor()
+        if actor:
+            return actor.monitors.get(self.name)
+         
     @property
     def ioqueue(self):
         if 'queue' not in self.local:
@@ -451,15 +425,12 @@ will have a :class:`IOLoop` instance based on the queue (via :class:`IOQueue`).
 By default it returns ``None``.'''
         return None
 
-    def put(self, request):
-        queue = self.ioqueue
-        if queue:
-            self.log.debug('Put {0} on IO queue'.format(request))
-            queue.put(('request',request))
-        else:
-            self.log.error("Trying to put a request on task queue,\
- but there isn't one!")
-
+    def on_config_init(self, cfg, params):
+        '''Callback when configuration is initialised but not yet loaded.
+This is a chance to add extra config parameter or remove unwanted ones.
+return a new :class:`Config` instance or ``None``.'''
+        pass
+    
     def on_config(self):
         '''Callback when configuration is loaded. This is a chance to do
  an application specific check before the concurrent machinery is put into
@@ -475,15 +446,6 @@ By default it returns ``None``.'''
 
     def add_timeout(self, deadline, callback):
         self.arbiter.ioloop.add_timeout(deadline, callback)
-
-    def get_config_options_include(self, params):
-        '''List of configuration option to include from in
-the Config dictionary'''
-        return self.config_options_include
-    
-    def get_config_options_exclude(self, params):
-        '''List of configuration option to exclude from the Config dictionary'''
-        return self.config_options_exclude
     
     def load_config(self, argv, version, parse_console, params):
         '''Load the application configuration from a file and/or
@@ -504,12 +466,15 @@ The parameters overriding order is the following:
         cfg_apps = set(self.cfg_apps or ())
         cfg_apps.add(self.app_name)
         self.cfg_apps = cfg_apps
-        self.cfg = pulsar.Config(self.description,
-                                 self.epilog,
-                                 version,
-                                 self.cfg_apps,
-                                 self.get_config_options_include(params),
-                                 self.get_config_options_exclude(params))
+        cfg = pulsar.Config(self.description,
+                            self.epilog,
+                            version,
+                            self.cfg_apps,
+                            self.config_options_include,
+                            self.config_options_exclude)
+        self.cfg = self.on_config_init(cfg, params)
+        if not isinstance(self.cfg, pulsar.Config):
+            self.cfg = cfg
         overrides = {}
         specials = set()
         # get the actor if available and override default cfg values with those
@@ -536,7 +501,7 @@ The parameters overriding order is the following:
                     self.cfg.settings[k].default = v
                 except AttributeError:
                     if not self.add_to_overrides(k, v, overrides):
-                        setattr(self,k,v)
+                        setattr(self, k, v)
         # parse console args
         if parse_console:
             parser = self.cfg.parser()
@@ -581,6 +546,10 @@ By default it returns ``None``.'''
     def on_event(self, worker, fd, events):
         pass
 
+    def worker_init(self, worker):
+        '''Callback by :class:`Worker` when initializing.'''
+        pass
+    
     def worker_start(self, worker):
         '''Called by the :class:`Worker` :meth:`pulsar.Actor.on_start`
 :ref:`callback <actor-callbacks>` method.'''
@@ -630,35 +599,10 @@ The application is now in the arbiter but has not yet started.'''
     def start(self):
         '''Start the application if it wasn't already started.'''
         arbiter = pulsar.arbiter()
-        if self.name in arbiter.monitors:
+        if arbiter and self.name in arbiter.monitors:
             arbiter.start()
         return self
 
-    def stop(self):
-        '''Stop the application.'''
-        arbiter = pulsar.arbiter()
-        monitor = arbiter.get_monitor(self.mid)
-        if monitor:
-            monitor.stop()
-
-    def configure_logging(self, config = None):
-        """Set the logging configuration as specified by the
- :ref:`logconfig <setting-logconfig>` setting."""
-        self.loglevel = self.cfg.loglevel
-        config = config or self.cfg.logconfig
-        super(Application,self).configure_logging(config=config)
-
-    def actorlinks(self, links):
-        if not links:
-            raise StopIteration
-        else:
-            arbiter = pulsar.arbiter()
-            for name,app in links.items():
-                if app.mid in arbiter.monitors:
-                    monitor = arbiter.monitors[app.mid]
-                    monitor.actor_links[self.name] = self
-                    yield name, app
-
 
 class MultiApp:
     
