@@ -1,7 +1,8 @@
 import sys
 import inspect
+import logging
 
-from pulsar import LogginMixin, to_bytes, is_failure, log_failure, is_async,\
+from pulsar import to_bytes, is_failure, log_failure, is_async,\
                     as_failure, maybe_async, HttpException
 from pulsar.utils.tools import checkarity
 from pulsar.apps.wsgi import WsgiResponse, WsgiResponseGenerator
@@ -12,13 +13,13 @@ from .exceptions import *
 
 __all__ = ['RpcHandler', 'RpcMiddleware']
 
+LOGGER = logging.getLogger('pulsar.rpc')
 
 class RpcRequest(object):
 
     def __init__(self, environ, handler, method, func, args,
                 kwargs, id, version):
         self.environ = environ
-        self.log = handler.log
         self.handler = handler
         self.method = method
         self.func = func
@@ -75,7 +76,7 @@ class ResponseGenerator(WsgiResponseGenerator):
                                        request.version,
                                        result=result)
         except Exception as e:
-            handler.log.error('Could not serialize', exc_info=True)
+            LOGGER.error('Could not serialize', exc_info=True)
             status_code = 500
             result = handler.dumps(request.id,
                                    request.version,
@@ -124,10 +125,7 @@ Add a limited ammount of magic to RPC handlers.'''
         return make(cls, name, bases, attrs)
 
 
-BaseHandler = MetaRpcHandler('BaseRpcHandler',(LogginMixin,),{'virtual':True})
-
-
-class RpcHandler(BaseHandler):
+class RpcHandler(MetaRpcHandler('_RpcHandler', (object,), {'virtual': True})):
     '''The base class for rpc handlers.
 
 .. attribute:: content_type
@@ -152,7 +150,6 @@ class RpcHandler(BaseHandler):
         self.subHandlers = {}
         self.title = title or self.__class__.__name__
         self.documentation = documentation or ''
-        self.setlog(**kwargs)
         if subhandlers:
             for prefix,handler in subhandlers.items():
                 if inspect.isclass(handler):
@@ -190,14 +187,14 @@ for ``method``, ``kwargs`` are keyworded parameters for ``method``,
 
 
     def __getstate__(self):
-        d = super(RpcHandler,self).__getstate__()
+        d = self.__dict__.copy()
         if not self.isroot():
             # Avoid duplicating handlers
             d['_parent'] = True
         return d
 
     def __setstate__(self, state):
-        super(RpcHandler,self).__setstate__(state)
+        self.__dict__ = state
         for handler in self.subHandlers.values():
             handler._parent = self
 
