@@ -14,7 +14,7 @@ import json
 from timeit import default_timer
 
 import pulsar
-from pulsar import HttpClient
+from pulsar import HttpClient, is_async
 from pulsar.utils.structures import AttributeDictionary
 from pulsar.utils.security import gen_unique_id
 from pulsar.utils.httpurl import to_string, range
@@ -49,7 +49,7 @@ Design to comply with the `JSON-RPC 2.0`_ Specification.
     def get_method_and_args(self, data):
         '''Overrides the :meth:`RpcHandler:get_method_and_args` to obtain
 method data from the JSON *data* string.'''
-        if not isinstance(data,dict):
+        if not isinstance(data, dict):
             data = self._json.loads(data)
         method  = data.get('method',None)
         params  = data.get('params',None)
@@ -57,8 +57,8 @@ method data from the JSON *data* string.'''
         version = data.get('jsonrpc',None)
         kwargs  = {}
         args    = ()
-        if isinstance(params,dict):
-            for k,v in params.items():
+        if isinstance(params, dict):
+            for k, v in params.items():
                 kwargs[str(k)] = v
         elif params:
             args = tuple(params)
@@ -199,6 +199,12 @@ usage is simple::
         # Always make sure the content-type is application/json
         self.http.headers['content-type'] = 'application/json'
         resp = self.http.post(self.__url, data=body)
+        if is_async(resp):
+            return resp.add_callback(lambda r: self._end_call(r, raw))
+        else:
+            return self._end_call(resp, raw)
+    
+    def _end_call(self, resp, raw):
         content = resp.content.decode('utf-8')
         if resp.is_error:
             if 'error' in content:
