@@ -188,20 +188,6 @@ from .scheduler import Scheduler
 from .states import *
 from .rpc import *
 
-def validate_list(val):
-    if isinstance(val,list):
-        return val
-    elif isinstance(val,tuple):
-        return list(val)
-    else:
-        val = to_string(val).split(',')
-        vals = []
-        for v in to_string(val).split(','):
-            v = v.strip()
-            if v:
-                vals.append(v)
-        return vals
-
 
 class TaskQueueFactory(pulsar.Setting):
     app = 'cpubound'
@@ -224,7 +210,7 @@ class TaskPath(TaskSetting):
     name = "tasks_path"
     section = "Task Consumer"
     meta = "STRING"
-    validator = validate_list
+    validator = pulsar.validate_list
     cli = ["--tasks-path"]
     default = ['pulsar.apps.tasks.testing']
     desc = """\
@@ -299,6 +285,12 @@ def job_list(client, actor, jobnames=None):
 def next_scheduled(client, actor, jobnames=None):
     return actor.app.scheduler.next_scheduled(jobnames=jobnames)
 
+@pulsar.command(commands_set=taskqueue_cmnds)
+def wait_for_task(client, actor, id, timeout=3600):
+    # wait for a task to finish for at most timeout seconds
+    scheduler = actor.app.scheduler
+    return scheduler.task_class.wait_for_task(scheduler, id, timeout)
+
 
 class TaskQueue(CPUboundServer):
     '''A :class:`pulsar.CPUboundServer` for consuming
@@ -350,7 +342,7 @@ to check if the scheduler needs to perform a new run.'''
         if self.callable:
             self.callable()
         import_modules(self.cfg.tasks_path)
-        self.local.scheduler = Scheduler(self.task_class)
+        self.local.scheduler = Scheduler(self)
         return self
 
     def monitor_handler(self):
