@@ -10,20 +10,38 @@ from threading import current_thread
 
 
 from pulsar import AlreadyCalledError, AlreadyRegistered,\
-                   ActorAlreadyStarted, LogSelf, LogginMixin, system,\
-                   Config
-
-from .eventloop import IOLoop, ID
+                   ActorAlreadyStarted, LogginMixin, system, Config
+from pulsar.utils.structures import AttributeDictionary
+from .eventloop import IOLoop, setid
 from .proxy import ActorProxy, ActorMessage, get_command, get_proxy
 from .defer import make_async, is_failure, iteritems, itervalues,\
-                     pickle, safe_async, async, log_failure, make_async
+                     pickle, safe_async, log_failure, make_async, is_async,\
+                     EXIT_EXCEPTIONS
 from .mailbox import IOQueue, mailbox
-from .access import set_local_data, is_mainthread, get_actor
+from .access import set_local_data, is_mainthread, get_actor, remove_actor
 
 
-__all__ = ['is_actor', 'send', 'Actor']
-
+__all__ = ['is_actor', 'send', 'Actor', 'ACTOR_STATES', 'Pulsar']
 
+ACTOR_STATES = AttributeDictionary(INITIAL=0X0, STARTING=0X1, RUN=0x2,
+                                   STOPPING=0x3, CLOSE=0x4, TERMINATE=0x5)
+ACTOR_STATES.DESCRIPTION = {ACTOR_STATES.INITIAL: 'initial',
+                            ACTOR_STATES.STARTING: 'starting',
+                            ACTOR_STATES.RUN: 'running',
+                            ACTOR_STATES.STOPPING: 'stopping',
+                            ACTOR_STATES.CLOSE: 'closed',
+                            ACTOR_STATES.TERMINATE:'terminated'}
+MINIMUM_ACTOR_TIMEOUT = 10
+DEFAULT_ACTOR_TIMEOUT = 60
+ACTOR_NOTIFY_PERIOD = 10
+# Send messages to arbiter not before the difference of
+# now and the last message time is greater than this tolerance
+# times timeout. So for a timeout of 30 seconds, the messages will
+# go after tolerance*30 seconds (18 secs for tolerance = 0.6).
+ACTOR_TIMEOUT_TOLERANCE = 0.6
+# Timeout of when joining an actor which has been terminated
+ACTOR_TERMINATE_TIMEOUT = 2
+ACTOR_STOPPING_LOOPS = 5
 EMPTY_TUPLE = ()
 EMPTY_DICT = {}
 
@@ -56,7 +74,21 @@ Typical example::
     return get_actor().send(target, action, *args, **params)
 
 
-class Actor(ID, LogginMixin):
+class Pulsar(LogginMixin):
+    
+    def configure_logging(self, **kwargs):
+        configure_logging = super(Pulsar, self).configure_logging
+        configure_logging(logger='pulsar',
+                          config=self.cfg.logconfig,
+                          level=self.cfg.loglevel,
+                          handlers=self.cfg.loghandlers) 
+        configure_logging(logger='pulsar.%s' % self.name,
+                          config=self.cfg.logconfig,
+                          level=self.cfg.loglevel,
+                          handlers=self.cfg.loghandlers)
+        
+
+class Actor(Pulsar):
     '''The base class for concurrent programming in pulsar. In computer science,
 the **Actor model** is a mathematical model of concurrent computation that
 treats *actors* as the universal primitives of computation.
@@ -107,8 +139,13 @@ Here ``a`` is actually a reference to the remote actor.
     An instance of :class:`IOLoop`, the input/output event loop
     driving the actor. Some actors may share the ioloop with other actors
     depending on their concurrency implementation.
+    
+.. attribute:: requestloop
 
-.. attribute:: nr
+    The :class:`IOLoop` to listen for requests. This is different from
+    :attr:`ioloop` for CPU-bound actors.
+
+.. attribute:: request_processed
 
     The total number of requests served by the actor
 
@@ -122,60 +159,16 @@ Here ``a`` is actually a reference to the remote actor.
 
     String indicating the logging level for the actor.
 '''
-    INITIAL = 0X0
-    STARTING = 0X1
-    RUN = 0x2
-    STOPPING = 0x3
-    CLOSE = 0x4
-    TERMINATE = 0x5
-    STATE_DESCRIPTION = {0x0:'initial',
-                         0x1:'starting',
-                         0x2:'running',
-                         0x3:'stopping',
-                         0x4:'closed',
-                         0x5:'terminated'}
-    DEFAULT_IMPLEMENTATION = 'process'
-    MINIMUM_ACTOR_TIMEOUT = 10
-    DEFAULT_ACTOR_TIMEOUT = 60
-    ACTOR_NOTIFY_PERIOD = 10
-    # Send messages to arbiter not before the difference of
-    # now and the last message time is greater than this tolerance
-    # times timeout. So for a timeout of 30 seconds, the messages will
-    # go after tolerance*30 seconds (18 secs for tolerance = 0.6).
-    ACTOR_TIMEOUT_TOLERANCE = 0.6
-    DEF_PROC_NAME = 'pulsar'
-    SIG_QUEUE = None
     exit_code = None
-
-    def __init__(self, impl, arbiter=None, monitor=None,
-                 on_task=None, ioqueue=None, monitors=None,
-                 name=None, params=None, age=0,
-                 pool_timeout=None, ppid=None, on_event=None,
-                 linked_actors=None, cfg=None, **kwargs):
-        self.cfg = cfg
-        self._ppid = ppid
-        self._impl = impl
-        self._proxy_mailboxes = {}
-        self._mailbox = None
-        self._linked_actors = linked_actors or {}
-        self.age = age
-        self.nr = 0
-        self.concurrent_requests = 0
-        self._pool_timeout = pool_timeout
-        self._name = (name or self.__class__.__name__).lower()
-        if self._name != self.aid:
-            self._repr = '%s %s' % (self._name, self.aid)
-        else:
-            self._repr = self._name
-        self.arbiter = arbiter
-        self.monitor = monitor
-        self._state = self.INITIAL
-        if impl.loglevel is None and arbiter:
-            self.loglevel = arbiter.loglevel
-        else:
-            self.loglevel = impl.loglevel
-        self._params = params or {}
-        self._monitors = monitors or {}
+    mailbox = None
+    stopping_start = None
+    stopping_end = None
+    
+    def __init__(self, impl):
+        self.__impl = impl
+        on_task = impl.params.pop('on_task', None)
+        on_event = impl.params.pop('on_event', None)
+        ioqueue = impl.params.pop('ioqueue', None)
         if not self.is_arbiter():
             if on_task:
                 self.on_task = lambda : on_task(self)
@@ -183,14 +176,46 @@ Here ``a`` is actually a reference to the remote actor.
                 self.on_event = lambda fd, event: on_event(self, fd, event)
         else:
             ioqueue = None
+        self.request_processed = 0
+        self.last_notified = 0
+        self.concurrent_requests = 0
+        self.state = ACTOR_STATES.INITIAL
         self.ioqueue = ioqueue
-        kwargs = self.on_init(**kwargs)
-        if kwargs:
-            self.local.update(kwargs)
-        if self.cfg is None:
-            self.cfg = {}
+        self.linked_actors = impl.params.pop('linked_actors', {})
+        self.monitors = impl.params.pop('monitors', {})
+        self.arbiter = impl.params.pop('arbiter', None)
+        self.monitor = impl.params.pop('monitor', None)
+        self.proxy_mailboxes = {}
+        self.params = AttributeDictionary(self.on_init(**impl.params))
+        del impl.params
 
+    def __repr__(self):
+        return self.impl.unique_name
+    
+    def __str__(self):
+        return self.__repr__()
+    
     ############################################################### PROPERTIES
+    @property
+    def impl(self):
+        return self.__impl
+    
+    @property
+    def aid(self):
+        return self.__impl.aid
+    
+    @property
+    def cfg(self):
+        return self.__impl.cfg
+    
+    @property
+    def name(self):
+        return self.__impl.name
+    
+    @property
+    def commands_set(self):
+        return self.__impl.commands_set
+    
     @property
     def proxy(self):
         '''Instance of a :class:`ActorProxy` holding a reference
@@ -198,93 +223,31 @@ to the actor. The proxy is a lightweight representation of the actor
 which can be shared across different processes (i.e. it is pickable).'''
         return ActorProxy(self)
 
-    @property
-    def aid(self):
-        '''Actor unique identifier'''
-        return self._impl.aid
-
-    @property
-    def ppid(self):
-        '''Parent process id.'''
-        return self._ppid
-
-    @property
-    def impl(self):
-        '''String indicating actor concurrency implementation
-("monitor", "thread", "process").'''
-        return self._impl.impl
-
-    @property
-    def commands_set(self):
-        return self._impl.commands_set
-
-    @property
-    def timeout(self):
-        '''Timeout in seconds. If ``0`` the actor has no timeout, otherwise
-it will be stopped if it fails to notify itself for a period
-longer that timeout.'''
-        return self._impl.timeout
-
-    @property
-    def mailbox(self):
-        '''Messages inbox :class:`Mailbox`.'''
-        return self._mailbox
-
     @property
     def address(self):
-        return self._mailbox.address
+        return self.mailbox.address
 
     @property
     def ioloop(self):
         '''The :class:`IOLoop` for I/O.'''
         return self.mailbox.ioloop
 
-    @property
-    def requestloop(self):
-        '''The :class:`IOLoop` to listen for requests. This is different from
- :attr:`ioloop` for CPU-bound actors.'''
-        return self._requestloop
-
-    @property
-    def monitors(self):
-        '''Dictionary of all :class:`Monitor` instances
-registered with the actor. The keys are given by the monitor names rather than
-their ids.'''
-        return self._monitors
-
     @property
     def cpubound(self):
-        if self.impl != 'monitor':
+        if self.impl.kind != 'monitor':
             return self.ioqueue is not None
         else:
             return False
 
     @property
-    def pool_timeout(self):
-        '''Timeout in seconds for waiting for events in the eventloop.
- A small number is suitable for :class:`Actor` performing CPU-bound
- operation on the :meth:`Actor.on_task` method, a larger number is better for
- I/O bound actors.
-'''
-        return self.requestloop.POLL_TIMEOUT
-
-    @property
-    def state(self):
+    def info_state(self):
         '''Current state description. One of ``initial``, ``running``,
  ``stopping``, ``closed`` and ``terminated``.'''
-        return self.STATE_DESCRIPTION[self._state]
+        return ACTOR_STATES.DESCRIPTION[self.state]
 
     ############################################################################
     ##    HIGH LEVEL API METHODS
     ############################################################################
-    def get(self, parameter, default=None):
-        '''retrive *parameter* form this :class:`Actor`.'''
-        return self._params.get(parameter, default)
-
-    def set(self, parameter, value):
-        '''Set *parameter* value on this :class:`Actor`.'''
-        self._params[parameter] = value
-
     def command(self, action):
         '''Fetch the pulsar command for *action*.'''
         return get_command(action, self.commands_set)
@@ -293,52 +256,39 @@ their ids.'''
         '''Send a message to *target* to perform *action* with given
 parameters *params*. It return a :class:`ActorMessage`.'''
         if not isinstance(target, ActorProxy):
-            tg = self.get_actor(target)
-        else:
-            tg = target
-        return get_proxy(tg).receive_from(self, action, *args, **params)
+            target = get_proxy(self.get_actor(target))
+        return target.receive_from(self, action, *args, **params)
 
     def put(self, request):
         '''Put a *request* into the :attr:`ioqueue` if available.'''
         if self.ioqueue:
-            self.log.debug('Putting %s into IO queue', request)
+            self.logger.debug('Putting %s into IO queue', request)
             self.ioqueue.put(('request', request))
         else:
-            self.log.error("Trying to put a request on task queue,\
+            self.logger.error("Trying to put a request on task queue,\
  but there isn't one!")
-
-    def run_on_arbiter(self, callable):
-        '''Run a *callable* in the arbiter event loop.
-
-:parameter callable: a pickable, therefore it must be a pickable callable object
-    or a function.
-:rtype: a :class:`Deferred`'''
-        return self.send('arbiter', 'run', callable)
-
+            
     ###############################################################  STATES
     def running(self):
         '''``True`` if actor is running.'''
-        return self._state == self.RUN
+        return self.state == ACTOR_STATES.RUN
 
     def started(self):
         '''``True`` if actor has started. It does not necessarily
 mean it is running.'''
-        return self._state >= self.RUN
+        return self.state >= ACTOR_STATES.RUN
 
     def stopping(self):
         '''``True`` if actor is stopping.'''
-        return self._state == self.STOPPING
+        return self.state == ACTOR_STATES.STOPPING
 
     def closed(self):
         '''``True`` if actor has exited in an clean fashion.'''
-        return self._state == self.CLOSE
+        return self.state == ACTOR_STATES.CLOSE
 
     def stopped(self):
         '''``True`` if actor has exited.'''
-        return self._state >= self.CLOSE
-
-    def is_pool(self):
-        return False
+        return self.state >= ACTOR_STATES.CLOSE
 
     def is_arbiter(self):
         '''Return ``True`` if ``self`` is the :class:`Arbiter`.'''
@@ -350,15 +300,15 @@ mean it is running.'''
 
     def isprocess(self):
         '''boolean indicating if this is an actor on a child process.'''
-        return self.impl == 'process'
+        return self.impl.kind == 'process'
 
     def ready(self):
-        return self.arbiter.aid in self._linked_actors
+        return self.arbiter.aid in self.linked_actors
 
     def can_poll(self):
         '''Check if the actor can poll requests. This is used by CPU-bound
  actors only.'''
-        m = self.cfg.get('backlog', 0)
+        m = self.cfg.backlog
         return self.concurrent_requests < m if m else True
 
     def __reduce__(self):
@@ -372,7 +322,7 @@ mean it is running.'''
         '''This function should be used when registering events
  on file descriptors. It is called by the :attr:`requestloop` when an event
  occurs on file descriptor *fd*.'''
-        self.nr += 1
+        self.request_processed += 1
         self.concurrent_requests += 1
         msg = safe_async(self.on_event, args=(fd, event))
         msg.addBoth(self.end_event)
@@ -380,10 +330,10 @@ mean it is running.'''
     def end_event(self, result):
         self.concurrent_requests -= 1
         log_failure(result)
-        max_requests = self.cfg.get('max_requests')
-        should_stop = max_requests and self.nr >= max_requests
+        max_requests = self.cfg.max_requests
+        should_stop = max_requests and self.request_processed >= max_requests
         if should_stop:
-            self.log.info("Auto-restarting worker.")
+            self.logger.info("Shutting down %s. Max requests reached.", self)
             self.stop()
 
     ############################################################################
@@ -392,7 +342,7 @@ mean it is running.'''
     def on_init(self, **kwargs):
         '''The :ref:`actor callback <actor-callbacks>` run once at the
 end of initialisation (after forking).'''
-        pass
+        return kwargs
 
     def on_start(self):
         '''The :ref:`actor callback <actor-callbacks>` run once just before
@@ -435,62 +385,67 @@ iteration of the :attr:`Actor.ioloop`.'''
         '''Called after forking to start the actor's life. This is where
 logging is configured, the :attr:`Actor.mailbox` is registered and the
 :attr:`Actor.ioloop` is initialised and started.'''
-        if self._state == self.INITIAL:
-            self._state = self.STARTING
+        if self.state == ACTOR_STATES.INITIAL:
+            self.state = ACTOR_STATES.STARTING
             self.configure_logging()
-            # wrap the logger
-            #if self.arbiter:
-            #    self.setlog(log=LogSelf(self,self.log))
             self._run()
 
     ############################################################################
     ##    INTERNALS
     ############################################################################
     def proxy_mailbox(self, address):
-        m = self._proxy_mailboxes.get(address)
+        m = self.proxy_mailboxes.get(address)
         if not m:
             m = mailbox(address=address)
-            self._proxy_mailboxes[address] = m
+            self.proxy_mailboxes[address] = m
         return m
 
     ############################################################################
     # STOPPING
     ############################################################################
-    @async()
-    def stop(self, force=False, exit_code=0):
+    def stop(self, force=False, exit_code=None):
         '''Stop the actor by stopping its :attr:`Actor.requestloop`
 and closing its :attr:`Actor.mailbox`. Once everything is closed
 properly this actor will go out of scope.'''
-        if force or self._state in (self.STARTING, self.RUN):
-            self.set('stopping_start', time())
-            self._state = self.STOPPING
+        if force or self.state in (ACTOR_STATES.STARTING, ACTOR_STATES.RUN):
+            self.stopping_start = time()
+            self.state = ACTOR_STATES.STOPPING
             self.exit_code = exit_code
-            # make safe user defined callbacks
-            yield safe_async(self.on_stop)
-            yield self.mailbox.close()
-            self._state = self.CLOSE
+            try:
+                res = self.on_stop()
+            except:
+                self.logger.error('Unhandle error while stopping',
+                                  exc_info=True)
+                res = None
+            return res.addBoth(self.exit) if is_async(res) else self.exit()
+    
+    def exit(self, result=None):
+        '''Exit from the :class:`Actor` domain.'''
+        if not self.stopped():
+            self.mailbox.close()
+            # we don't want monitors to stop the request loop
             if not self.is_monitor():
                 self.requestloop.stop()
+            self.state = ACTOR_STATES.CLOSE
+            self.stopping_end = time()
+            self.logger.debug('%s exited', self)
+            remove_actor(self)
             self.on_exit()
-            self.set('stopping_end', time())
-            self.log.info('Exited')
-
-    def linked_actors(self):
-        '''Iterator over linked-actor proxies.'''
-        for a in itervalues(self._linked_actors):
-            if isinstance(a, ActorProxy):
-                yield a
 
     def get_actor(self, aid):
         '''Given an actor unique id return the actor proxy.'''
         if aid == self.aid:
             return self
         elif aid == 'arbiter':
+            return self.arbiter or self
+        elif self.arbiter and aid == self.arbiter.aid:
             return self.arbiter
-        elif aid in self._linked_actors:
-            return self._linked_actors[aid]
+        elif self.monitor and aid == self.monitor.aid:
+            return self.monitor
+        elif aid in self.linked_actors:
+            return self.linked_actors[aid]
         else:
-            return self._monitors.get(aid)
+            return self.monitors.get(aid)
 
     def __call__(self):
         '''Called in the main eventloop to perform the following
@@ -499,18 +454,16 @@ actions:
 * it notifies linked actors if required (arbiter only for now)
 * it executes the :meth:`Actor.on_task` callback.
 '''
-        if self.running() and self.arbiter.aid in self._linked_actors:
+        if self.running() and self.arbiter.aid in self.linked_actors:
             nt = time()
             if hasattr(self, 'last_notified'):
-                timeout = self.timeout or self.DEFAULT_ACTOR_TIMEOUT
-                tole = min(self.ACTOR_TIMEOUT_TOLERANCE*timeout,
-                           self.ACTOR_NOTIFY_PERIOD)
+                timeout = self.cfg.timeout
+                tole = min(ACTOR_TIMEOUT_TOLERANCE*timeout, ACTOR_NOTIFY_PERIOD)
                 if nt - self.last_notified < tole:
                     nt = None
             if nt:
                 self.last_notified = nt
                 info = self.info()
-                info['actor']['last_notified'] = nt
                 self.send('arbiter', 'notify', info)
             self.on_task()
 
@@ -522,15 +475,15 @@ status and performance.'''
         isp = self.isprocess()
         requestloop  = self.requestloop
         actor = {'name': self.name,
-                 'actor_id': self.aid[:8],
+                 'state': self.info_state,
+                 'actor_id': self.aid,
                  'uptime': time() - requestloop._started,
-                 'ppid': self.ppid,
                  'thread_id': self.tid,
                  'process_id': self.pid,
                  'is_process': isp,
                  'internal_connections': self.mailbox.active_connections,
-                 'age': self.age}
-        events = {'request processed': self.nr,
+                 'age': self.impl.age}
+        events = {'request processed': self.request_processed,
                   'callbacks': len(self.ioloop._callbacks),
                   'io_loops': self.ioloop.num_loops}
         if self.cpubound:
@@ -550,7 +503,7 @@ if *proxy* is not a class:`ActorProxy` instance raise an exception.'''
             proxy.address = address
         if not proxy.address:
             raise ValueError('Linking with a actor without address')
-        self._linked_actors[proxy.aid] = proxy
+        self.linked_actors[proxy.aid] = proxy
         if proxy.aid == self.arbiter.aid:
             self.requestloop.ready = True
         # If the proxy is the actor monitor, add the arbiter
@@ -566,18 +519,14 @@ if *proxy* is not a class:`ActorProxy` instance raise an exception.'''
         if not self.isprocess():
             return
         random.seed()
-        proc_name = self.DEF_PROC_NAME
-        cfg = self.cfg
-        proc_name = cfg.get('proc_name') or cfg.get('default_proc_name')\
-                         or proc_name
-        proc_name = "{0} - {1}".format(proc_name, self)
+        proc_name = "%s-%s" % (self.cfg.proc_name, self)
         if system.set_proctitle(proc_name):
-            self.log.debug('Set process title to %s', proc_name)
+            self.logger.debug('Set process title to %s', proc_name)
         #system.set_owner_process(cfg.uid, cfg.gid)
         if is_mainthread():
-            self.log.debug('Installing signals')
+            self.logger.debug('Installing signals')
             # The default signal handling function in signal
-            sfun = getattr(self,'signal',None)
+            sfun = getattr(self, 'signal', None)
             for name in system.ALL_SIGNALS:
                 func = sfun
                 if not func:
@@ -595,41 +544,28 @@ if *proxy* is not a class:`ActorProxy` instance raise an exception.'''
 event loop which will consume events on file descriptors.'''
         # Inject self as the actor of this thread
         ioq = self.ioqueue
-        self._requestloop = IOLoop(io=IOQueue(ioq, self) if ioq else None,
-                                   pool_timeout=self._pool_timeout,
-                                   logger=self.log,
-                                   name=self.name,
-                                   ready=not self.cpubound)
+        self.requestloop = IOLoop(io=IOQueue(ioq, self) if ioq else None,
+                                  poll_timeout=self.params.poll_timeout,
+                                  logger=self.logger,
+                                  ready=not self.cpubound)
         # If CPU bound add the request handler to the request loop
         if self.cpubound:
-            self._requestloop.add_handler('request',
+            self.requestloop.add_handler('request',
                                           self.handle_fd_event,
-                                          self._requestloop.READ)
+                                          self.requestloop.READ)
         self._init_runner()
-        self._mailbox = mailbox(self)
+        self.mailbox = mailbox(self)
         set_local_data(self)
-        self.setid()
-        self._state = self.RUN
+        setid(self)
+        self.state = ACTOR_STATES.RUN
         self.on_start()
-        self.log.info('Address %s', self.mailbox.address)
+        self.logger.info('address %s', self.mailbox.address)
 
     def _run(self):
         '''The run implementation which must be done by a derived class.'''
         self._on_run()
         try:
             self.requestloop.start()
-        except:
-            self.log.critical("Unhandled exception exiting.", exc_info = True)
         finally:
             self.stop()
-
-
-    def _signal_stop(self, sig, frame):
-        signame = system.SIG_NAMES.get(sig,None)
-        self.log.warning('got signal {0}. Exiting.'.format(signame))
-        self.stop()
-
-    handle_int  = _signal_stop
-    handle_quit = _signal_stop
-    handle_term = _signal_stop
-    handle_break = _signal_stop
\ No newline at end of file
+    
\ No newline at end of file
