@@ -11,9 +11,10 @@ from pulsar.utils.tools import Pidfile
 from pulsar.utils.security import gen_unique_id
 from pulsar import HaltServer
 
-from .defer import itervalues, iteritems, multi_async
-from .actor import Actor, send
-from .monitor import PoolMixin
+from .defer import itervalues, iteritems, multi_async, async
+from .actor import Actor, ACTOR_STATES,  ACTOR_TERMINATE_TIMEOUT,\
+                     ACTOR_STOPPING_LOOPS
+from .monitor import PoolMixin, _spawn_actor
 from .access import get_actor, set_actor
 from . import proxy
 
@@ -29,16 +30,20 @@ process_global = pulsar.process_global
 __all__ = ['arbiter', 'spawn', 'Arbiter', 'ThreadQueue']
 
 
-def arbiter(daemonize=False):
+def arbiter(commands_set=None, **params):
     '''Obtain the arbiter instance.'''
     arbiter = get_actor()
     if arbiter is None:
-        return set_actor(Arbiter.make(daemonize=daemonize))
+        # Create the arbiter
+        cset = set(proxy.actor_commands)
+        cset.update(proxy.arbiter_commands)
+        cset.update(commands_set or ())
+        return set_actor(_spawn_actor(Arbiter, commands_set=cset, **params))
     elif isinstance(arbiter, Actor) and arbiter.is_arbiter():
         return arbiter
 
 
-def spawn(**kwargs):
+def spawn(cfg=None, **kwargs):
     '''Spawn a new :class:`Actor` and return an :class:`ActorProxyDeferred`.
 This method can be used from any :class:`Actor`.
 If not in the :class:`Arbiter` domain,
@@ -72,8 +77,8 @@ A typical usage::
     # The actor is not the Arbiter domain.
     # We send a message to the Arbiter to spawn a new Actor
     if not isinstance(actor, Arbiter):
-        msg = send('arbiter', 'spawn', **kwargs)\
-                        .add_callback(actor.link_actor)
+        msg = actor.send('arbiter', 'spawn', **kwargs)\
+                            .add_callback(actor.link_actor)
         return proxy.ActorProxyDeferred(aid, msg)
     else:
         return actor.spawn(**kwargs)
@@ -96,12 +101,10 @@ Users access the arbiter by the high level api::
 .. _twisted: http://twistedmatrix.com/trac/
 .. _tornado: http://www.tornadoweb.org/
 '''
-    STOPPING_LOOPS = 20
-    SIG_TIMEOUT = 0.01
-    EXIT_SIGNALS = (signal.SIGINT,
-                    signal.SIGTERM,
-                    signal.SIGABRT,
-                    system.SIGQUIT)
+    pidfile = None
+    restarted = False
+    EXIT_SIGNALS = (signal.SIGINT, signal.SIGTERM,
+                    signal.SIGABRT, system.SIGQUIT)
 
     ############################################################################
     # ARBITER HIGH LEVEL API
@@ -109,78 +112,56 @@ Users access the arbiter by the high level api::
     def is_arbiter(self):
         return True
 
-    def add_monitor(self, monitor_class, monitor_name, **kwargs):
+    def add_monitor(self, monitor_class, monitor_name, **params):
         '''Add a new :class:`Monitor` to the :class:`Arbiter`.
 
 :parameter monitor_class: a :class:`pulsar.Monitor` class.
 :parameter monitor_name: a unique name for the monitor.
 :parameter kwargs: dictionary of key-valued parameters for the monitor.
 :rtype: an instance of a :class:`pulsar.Monitor`.'''
-        if monitor_name in self._monitors:
+        if monitor_name in self.monitors:
             raise KeyError('Monitor "{0}" already available'\
                            .format(monitor_name))
-        kwargs['name'] = monitor_name
-        kwargs['aid'] = monitor_name
-        m = arbiter().spawn(monitor_class, **kwargs)
-        # Monitors don't hold other monitors
-        m._monitors.clear()
-        self._linked_actors[m.aid] = m
-        self._monitors[m.name] = m
+        params['name'] = monitor_name
+        m = self.spawn(monitor_class, **params)
+        self.linked_actors[m.aid] = m
+        self.monitors[m.name] = m
         return m
 
-    @classmethod
-    def make(cls, commands_set=None, impl=None, **kwargs):
-        commands_set = set(commands_set or proxy.actor_commands)
-        commands_set.update(proxy.arbiter_commands)
-        return cls._spawn_actor(None, cls, 'arbiter', commands_set, **kwargs)
-
-    @property
-    def close_signal(self):
-        '''Return the signal that caused the arbiter to stop.'''
-        return self._close_signal
-
     def isprocess(self):
         return True
 
     def get_all_monitors(self):
         '''A dictionary of all :class:`Monitor` in the arbiter'''
-        return dict(((mon.name,mon.proxy) for mon in itervalues(self.monitors)))
+        return dict(((mon.name, mon.proxy) for mon in\
+                      itervalues(self.monitors) if mon.mailbox))
 
     @multi_async
     def close_monitors(self):
         '''Close all :class:`Monitor` at once.'''
-        for pool in list(itervalues(self._monitors)):
-            yield pool.stop()
+        for pool in list(itervalues(self.monitors)):
+            yield pool.stop(True)
 
     def on_info(self, data):
         monitors = [p.info() for p in itervalues(self.monitors)]
         server = data.pop('actor')
-        server.update({'version':pulsar.__version__,
-                       'name':pulsar.SERVER_NAME,
-                       'number_of_monitors':len(self._monitors),
-                       'number_of_actors':len(self.MANAGED_ACTORS)})
+        server.update({'version': pulsar.__version__,
+                       'name': pulsar.SERVER_NAME,
+                       'number_of_monitors': len(self.monitors),
+                       'number_of_actors': len(self.managed_actors)})
         server.pop('is_process', None)
         server.pop('ppid', None)
         server.pop('actor_id', None)
         server.pop('age', None)
         data['server'] = server
-        data['workers'] = [a.info for a in itervalues(self.MANAGED_ACTORS)]
+        data['workers'] = [a.info for a in itervalues(self.managed_actors)]
         data['monitors'] = monitors
         return data
 
-    def configure_logging(self, config = None):
-        if self._monitors:
-            monitor = list(self._monitors.values())[0]
-            monitor.configure_logging(config = config)
-            self.setlog()
-            self.loglevel = monitor.loglevel
-        else:
-            super(Arbiter,self).configure_logging(config = config)
-
     ############################################################################
     # OVERRIDE ACTOR HOOKS
     ############################################################################
-    def on_init(self, daemonize=False, **kwargs):
+    def on_init(self, **kwargs):
         testing = kwargs.pop('__test_arbiter__',False)
         if not testing:
             if current_process().daemon:
@@ -189,32 +170,30 @@ Users access the arbiter by the high level api::
             if isinstance(get_actor(), self.__class__):
                 raise pulsar.PulsarException('Arbiter already created')
             os.environ["SERVER_SOFTWARE"] = pulsar.SERVER_SOFTWARE
-        self._close_signal = None
-        if daemonize:
+        if self.cfg.daemon:
             system.daemonize()
-        self.actor_age = 0
-        self.reexec_pid = 0
-        self.SIG_QUEUE = ThreadQueue()
+        self.signal_queue = ThreadQueue()
         return PoolMixin.on_init(self, **kwargs)
 
     def on_start(self):
-        pidfile = self.get('pidfile')
+        pidfile = self.cfg.pidfile
         if pidfile is not None:
             p = Pidfile(pidfile)
             p.create(self.pid)
-            self.local.pidfile = p
+            self.pidfile = p
 
     def on_task(self):
         '''Override the :class:`Actor.on_task` callback to perform the
 arbiter tasks at every iteration in the event loop.'''
-        sig = self._arbiter_task()
-        if sig is None:
+        if self.restarted:
+            self.stop(exit_code=self.exit_code)
+        if not self._arbiter_task():
             self.manage_actors()
-            for m in list(itervalues(self._monitors)):
+            for m in list(itervalues(self.monitors)):
                 if m.started():
                     if not m.running():
-                        self.log.info('Removing monitor {0}'.format(m))
-                        self._monitors.pop(m.name)
+                        self.logger.info('Removing monitor %s', m)
+                        self.monitors.pop(m.name)
                 else:
                     m.start()
         try:
@@ -227,34 +206,41 @@ arbiter tasks at every iteration in the event loop.'''
 the timeout. Stop the arbiter.'''
         if self.running() and actor.notified:
             gap = time() - actor.notified
-            if gap > actor.timeout:
-                if actor.stopping_loops < self.STOPPING_LOOPS:
+            if gap > actor.cfg.timeout:
+                if actor.stopping_loops < ACTOR_STOPPING_LOOPS:
                     if not actor.stopping_loops:
-                        self.log.info(\
-                            'Stopping {0}. Timeout surpassed.'.format(actor))
+                        self.logger.info('Stopping %s. Timeout.', actor)
                         self.send(actor, 'stop')
+                    actor.stopping_loops += 1
                 else:
-                    self.log.warn(\
-                            'Terminating {0}. Timeout surpassed.'.format(actor))
+                    self.logger.warn('Terminating %s. Timeout.', actor)
                     actor.terminate()
-                    actor.join(self.JOIN_TIMEOUT)
-                actor.stopping_loops += 1
+                    actor.join(ACTOR_TERMINATE_TIMEOUT)
 
+    @async()
     def on_stop(self):
         '''Stop the pools the message queue and remaining actors.'''
-        # close all monitors
-        self.log.info('Shutting down server.')
-        yield self.close_monitors()
-        # close remaining actors
-        yield self.close_actors()
-        yield self._close_message_queue()
-
+        if not self.ioloop.running():
+            for pool in list(itervalues(self.monitors)):
+                pool.state = ACTOR_STATES.STOPPING
+            self.state = ACTOR_STATES.RUN
+            self.restarted = True
+            self.ioloop.start()
+        else:
+            self.state = ACTOR_STATES.STOPPING
+            yield self.close_monitors()
+            yield self.close_actors()
+            yield self._close_message_queue()
+    
     def on_exit(self):
-        p = self.get('pidfile')
+        self.logger.info("Shutting down server.")
+        p = self.pidfile
         if p is not None:
             p.unlink()
-        if self.MANAGED_ACTORS:
-            self._state = self.TERMINATE
+        if self.managed_actors:
+            self.state = ACTOR_STATES.TERMINATE
+        if self.exit_code is not None:
+            sys.exit(self.exit_code)
 
     ############################################################################
     # INTERNALS
@@ -266,34 +252,27 @@ the timeout. Stop the arbiter.'''
         """
         self._on_run()
         try:
-            self.cfg.get('when_ready')(self)
+            self.cfg.when_ready(self)
         except:
             pass
         try:
             self.ioloop.start()
-        except StopIteration:
-            self._halt('Stop Iteration')
-        except KeyboardInterrupt:
-            self._halt('Keyboard Interrupt')
         except HaltServer as e:
-            self._halt(reason=str(e), sig=e.signal)
-        except SystemExit:
-            raise
+            self._halt(reason=str(e), code=e.signal)
+        except (KeyboardInterrupt, SystemExit) as e:
+            self._halt(e.__class__.__name__)
         except:
-            self._halt("Unhandled exception in main loop.")
-        if self.exit_code:
-            sys.exit(self.exit_code)
+            self._halt(code=1)
 
-    def _halt(self, reason=None, sig=None):
+    def _halt(self, reason=None, code=None):
         #halt the arbiter. If there no signal ``sig`` it is an unexpected exit
-        x = 'Shutting down pulsar arbiter'
-        _msg = lambda : x if not reason else '{0}: {1}'.format(x,reason)
-        if not sig:
-            self.log.critical(_msg())
-        else:
-            self.log.info(_msg())
-        self._close_signal = sig
-        self.stop()
+        if not self.closed():
+            if not reason:
+                reason = "Unhandled exception in main loop."
+                self.logger.critical(reason, exc_info=True)
+            else:
+                self.logger.info(reason)
+            self.stop(True, exit_code=code)
 
     def _close_message_queue(self):
         return
@@ -304,40 +283,32 @@ signal queue'''
         sig = None
         while True:
             try:
-                sig = self.SIG_QUEUE.get(timeout=self.SIG_TIMEOUT)
-            except (Empty,IOError):
+                sig = self.signal_queue.get(timeout=0.01)
+            except (Empty, IOError):
                 break
             if sig not in system.SIG_NAMES:
-                self.log.info("Ignoring unknown signal: %s" % sig)
+                self.logger.info("Ignoring unknown signal: %s", sig)
                 sig = None
             else:
                 signame = system.SIG_NAMES.get(sig)
                 if sig in self.EXIT_SIGNALS:
-                    raise HaltServer('Received Signal {0}.'\
-                                          .format(signame),sig)
-                handler = getattr(self, "handle_queued_%s"\
-                                   % signame.lower(), None)
+                    raise HaltServer('Received Signal %s.' % signame, sig)
+                handler = getattr(self, "signal_%s" % signame.lower(), None)
                 if not handler:
-                    self.log.debug('Cannot handle signal "{0}". No Handle'\
-                                   .format(signame))
+                    self.logger.debug('No Handle for signal "%s".', signame)
                     sig = None
                 else:
-                    self.log.info("Handling signal: %s" % signame)
-                    handler()
+                    self.logger.info("Handling signal: %s", signame)
+                    sig = handler()
         return sig
 
     def signal(self, sig, frame=None):
         signame = system.SIG_NAMES.get(sig, None)
         if signame:
             if self.ioloop.running():
-                self.log.debug('Received and queueing signal {0}.'\
-                               .format(signame))
-                self.SIG_QUEUE.put(sig)
+                self.logger.debug('Received and queueing signal %s.', signame)
+                self.signal_queue.put(sig)
                 self.ioloop.wake()
-            else:
-                pass
-                #exit(1)
         else:
-            self.log.debug('Received unknown signal "{0}". Skipping.'\
-                          .format(sig))
+            self.logger.debug('Received unknown signal "%s". Skipping.', sig)
 
