@@ -16,11 +16,11 @@ from pulsar.utils.log import Synchronized
 from pulsar.utils.structures import WeakList
 
 from .defer import Deferred, is_async, maybe_async, thread_loop, make_async,\
-                    log_failure
+                    log_failure, EXIT_EXCEPTIONS
 
 __all__ = ['IOLoop', 'PeriodicCallback', 'loop_timeout']
 
-EXIT_EXCEPTIONS = (KeyboardInterrupt, SystemExit, HaltServer)
+LOGGER = logging.getLogger('pulsar.eventloop')
 
 def file_descriptor(fd):
     if hasattr(fd,'fileno'):
@@ -28,40 +28,9 @@ def file_descriptor(fd):
     else:
         return fd
 
-
-class ID:
-    cpubound = False
-    _name = None
-    _pid = None
-    _repr = ''
-    _tid = None
-
-    def __repr__(self):
-        return self._repr
-
-    def __str__(self):
-        return self._repr
-
-    @property
-    def tid(self):
-        return self._tid
-
-    @property
-    def pid(self):
-        return self._pid
-
-    @property
-    def name(self):
-        return self._name
-
-    @property
-    def fullname(self):
-        return self._repr
-
-    def setid(self):
-        ct = current_thread()
-        self._tid = ct.ident
-        self._pid = os.getpid()
+def setid(self):
+    self.tid = current_thread().ident
+    self.pid = os.getpid()
 
 
 class LoopGuard(object):
@@ -71,24 +40,22 @@ class LoopGuard(object):
 
     def __enter__(self):
         loop = self.loop
-        loop.log.debug("Starting %s", loop)
+        loop.logger.debug("Starting event loop")
         loop._running = True
-        loop.setid()
-        loop._started = time.time()
+        if not loop._started:
+            loop._started = time.time()
+        setid(loop)
         loop._on_exit = Deferred()
-        loop.num_loops = 0
         return self
 
     def __exit__(self, type, value, traceback):
         loop = self.loop
         loop._running = False
-        loop._tid = None
-        loop._stopped = True
-        loop.log.debug('Exiting event loop')
+        loop.logger.debug('Exiting event loop')
         loop._on_exit.callback(loop)
 
 
-class IOLoop(IObase, ID, Synchronized):
+class IOLoop(IObase, Synchronized):
     """\
 A level-triggered I/O event loop adapted from tornado.
 
@@ -102,7 +69,7 @@ A level-triggered I/O event loop adapted from tornado.
 
     total number of loops
 
-.. attribute:: POLL_TIMEOUT
+.. attribute:: poll_timeout
 
     The timeout in seconds when polling with epol or select.
 
@@ -113,33 +80,22 @@ A level-triggered I/O event loop adapted from tornado.
     The thread id where the eventloop is running
 """
     # Never use an infinite timeout here - it can stall epoll
-    POLL_TIMEOUT = 0.5
+    poll_timeout = 0.5
 
-    def __init__(self, io=None, logger=None, pool_timeout=None, commnads=None,
-                 name=None, ready=True):
+    def __init__(self, io=None, logger=None, poll_timeout=None, ready=True):
         self._impl = io or IOpoll()
-        self.POLL_TIMEOUT = pool_timeout if pool_timeout is not None\
-                                 else self.POLL_TIMEOUT
-        self.log = logger or logging.getLogger('ioloop')
+        self.poll_timeout = poll_timeout if poll_timeout else self.poll_timeout
+        self.logger = logger or LOGGER
         if hasattr(self._impl, 'fileno'):
             close_on_exec(self._impl.fileno())
-        cname = self.__class__.__name__.lower()
-        self._name = name or cname
-        if self._name != cname:
-            self._repr = '%s - %s' % (cname, self._name)
-        else:
-            self._repr = self._name
         self._handlers = {}
         self._events = {}
         self._callbacks = []
         self._timeouts = []
         self._loop_tasks = WeakList()
-        self._starter = None
         self._started = None
         self._running = False
-        self._stopped = False
         self.num_loops = 0
-        self._blocking_signal_threshold = None
         self._waker = getattr(self._impl, 'waker', Waker)()
         self._on_exit = None
         self.ready = ready
@@ -179,7 +135,7 @@ file descriptor *fd*.
                 self._impl.register(fdd, events | self.ERROR)
                 return True
             else:
-                self.log.debug('Handler for %s already available.', fd)
+                self.logger.debug('Handler for %s already available.', fd)
 
     def update_handler(self, fd, events):
         """Changes the events we listen for fd."""
@@ -193,17 +149,12 @@ file descriptor *fd*.
         try:
             self._impl.unregister(fdd)
         except (OSError, IOError):
-            self.log.error("Error removing %s from IOLoop", fd, exc_info=True)
+            self.logger.error("Error removing %s from IOLoop", fd, exc_info=True)
 
-    def start(self, starter=None):
-        if not self._startup(starter):
+    def start(self):
+        if self._running:
             return False
-        if starter: # just start
-            self._run()
-        elif self._starter:
-            self._starter.start()
-        else:
-            self._run()
+        self._run()
 
     def stop(self):
         '''Stop the loop after the current event loop iteration is complete.
@@ -228,10 +179,6 @@ whether that callback was invoked before or after ioloop.start.'''
         """Returns true if this IOLoop is currently running."""
         return self._running
 
-    def stopped(self):
-        '''Return ``True`` if the loop have been stopped.'''
-        return self._stopped
-
     def add_timeout(self, deadline, callback):
         """Add a timeout *callback*. A timeout callback  it is called
 at the time *deadline* from the :class:`IOLoop`.
@@ -258,61 +205,45 @@ by the :meth:`add_timeout` method."""
         if wake:
             self.wake()
 
+    def add_periodic(self, callback, period):
+        """Add a :class:`PeriodicCallback` to the event loop."""
+        p = PeriodicCallback(callback, period, self)
+        p.start()
+        return p
+        
     def wake(self):
         '''Wake up the eventloop.'''
-        if not self.stopped():
+        if self.running():
             self._waker.wake()
 
-    ############################################################ INTERNALS
-    @Synchronized.make
-    def _startup(self, starter):
-        if self._running:
-            return False
-        if starter:
-            if self._starter and starter is not self._starter:
-                raise RuntimeError('Cannot start. It needs to be started '\
-                                   'by %s' % self._starter)
-            self._starter = starter
-        self._stopped = False
-        return True
-
-    def do_loop_tasks(self):
-        '''Perform tasks in the event loop. These tasks can be added and
-removed using the :meth:`pulsar.IOLoop.add_loop_task` and
-:meth:`pulsar.IOLoop.remove_loop_task` methods.
-
-For example, a :class:`pulsar.Actor` add itself to the event loop tasks
-so that it can perform its tasks at each event loop. Check the
-:meth:`pulsar.Actor` method.'''
-        for task in self._loop_tasks:
-            try:
-                result = task()
-                #result = task()
-                #loop_deferred(result, self)
-            except HaltServer:
-                raise
-            except:
-                self.log.critical('Unhandled exception in loop task',
-                                  exc_info = True)
+    ############################################################ INTERNALS    
+    def _run_callback(self, callback, name='callback'):
+        try:
+            callback()
+        except EXIT_EXCEPTIONS:
+            raise
+        except:
+            self.logger.critical('Unhandled exception in %s.', name,
+                                 exc_info=True)
 
     def _run(self):
         """Runs the I/O loop until one of the I/O handlers calls stop(), which
 will make the loop stop after the current event iteration completes."""
         with LoopGuard(self) as guard:
             while self._running:
-                poll_timeout = self.POLL_TIMEOUT
+                poll_timeout = self.poll_timeout
                 self.num_loops += 1
+                _run_callback = self._run_callback
+                # Loop task first
+                for task in self._loop_tasks:
+                    _run_callback(task, 'loop task')
                 # Prevent IO event starvation by delaying new callbacks
                 # to the next iteration of the event loop.
                 callbacks = self._callbacks
                 if callbacks:
                     self._callbacks = []
-                    #self.log.debug('Total of %s callbacks', len(callbacks))
-                    _run_callback = self._run_callback
                     for callback in callbacks:
                         _run_callback(callback)
-                #if self._callbacks:
-                #    poll_timeout = 0.0
                 if self._timeouts:
                     now = time.time()
                     while self._timeouts and self._timeouts[0].deadline <= now:
@@ -324,13 +255,9 @@ will make the loop stop after the current event iteration completes."""
                 # A chance to exit
                 if not self._running:
                     break
-                self.do_loop_tasks()
+                # Not ready to poll
                 if not self.ready:
                     continue
-                if self._blocking_signal_threshold is not None:
-                    # clear alarm so it doesn't fire while poll is waiting for
-                    # events.
-                    signal.setitimer(signal.ITIMER_REAL, 0, 0)
                 try:
                     event_pairs = self._impl.poll(poll_timeout)
                 except Exception as e:
@@ -345,9 +272,6 @@ will make the loop stop after the current event iteration completes."""
                         continue
                     else:
                         raise
-                if self._blocking_signal_threshold is not None:
-                    signal.setitimer(signal.ITIMER_REAL,
-                                     self._blocking_signal_threshold, 0)
                 # Pop one fd at a time from the set of pending fds and run
                 # its handler. Since that handler may perform actions on
                 # other file descriptors, there may be reentrant calls to
@@ -366,35 +290,15 @@ will make the loop stop after the current event iteration completes."""
                                 # Happens when the client closes the connection
                                 pass
                             else:
-                                self.log.error(
+                                self.logger.error(
                                     "Exception in I/O handler for fd %s",
                                               fd, exc_info=True)
                         except KeyError:
-                            self.log.info("File descriptor %s missing" % fd)
+                            self.logger.info("File descriptor %s missing", fd)
                         except:
-                            self.log.error("Exception in I/O handler for fd %s",
+                            self.logger.error("Exception in I/O handler for fd %s",
                                           fd, exc_info=True)
 
-    def _run_callback(self, callback):
-        try:
-            callback()
-        except EXIT_EXCEPTIONS:
-            raise
-        except:
-            self.handle_callback_exception(callback)
-
-    def handle_callback_exception(self, callback):
-        """This method is called whenever a callback run by the IOLoop
-        throws an exception.
-
-        By default simply logs the exception as an error.  Subclasses
-        may override this method to customize reporting of exceptions.
-
-        The exception itself is not passed explicitly, but is available
-        in sys.exc_info.
-        """
-        self.log.error("Exception in callback %r", callback, exc_info=True)
-
 
 class _Timeout(object):
     """An IOLoop timeout, a UNIX timestamp and a callback"""
@@ -432,11 +336,11 @@ class PeriodicCallback(object):
         if not self._running:
             return
         try:
-            self.callback()
+            self.callback(self)
         except (KeyboardInterrupt, SystemExit):
             raise
         except:
-            ioloop.log.error("Error in periodic callback", exc_info=True)
+            LOGGER.error("Error in periodic callback", exc_info=True)
         if self._running:
             self.start()
 
